;MultiKitB: AVR Oscilloscope and Development Kit
;
;Gabotronics C.A.
;March 2009
;
;Copyright 2009 Gabriel Anzziani
;
;This program is distributed under the terms of the GNU General Public License 
;
;www.gabotronics.com
;email me at: gabriel@gabotronics.com

#include <avr/io.h>

;----------------------------------------------------------------------------;
; 1:7 Fixed Point Multiplication
.global fmuls_8
.func fmuls_8
fmuls_8:
    push r23
    mov r23, r24
    fmuls R22, R23
    mov r24, r1
    pop r23
	clr	r1
	ret
.endfunc

; 8:8 Fixed Point Multiplication
.global fixmult
.func fixmult
fixmult:
    push r20
    push r21
    movw  r20,r24
    
    muls	r23, r21	; (signed)ah * (signed)bh
    mov		r31, r0		;
    mul		r22, r20	; al * bl
    mov 	r30, r1		;
    ;mov	r16, r0
    mulsu	r23, r20	; (signed)ah * bl
    add		r30, r0     ;
    adc		r31, r1     ;
    mulsu	r21, r22	; (signed)bh * al
    add		r30, r0     ;
    adc		r31, r1     ;
	
    movw    r24,r30
    pop r21
    pop r20
    clr r1
    ret
.endfunc


; falling edge trigger detect
;----------------------------------------------------------------------------;
;
;                 data1  ___.
;                            \
;  triggervalue - - - - - - - \
;                              \.___ data2
;
.global trigdown
.func trigdown
trigdown:	; input r24: triggervalue
	push	r26			; data
trigcheck1d:
	in		r26,0x06	; 1 cycle ; data1= Read portB input
/*    dec     r26         ; 1
    dec     r26         ; 1
    dec     r26         ; 1
    lpm*/
	cp		r24,r26		; 1 cycle ; compare triggervalue with data1
	brlo    trigcheck2d	; 1 cycle  ; triggervalue < data1 ?
	sbis	_SFR_IO_ADDR(GPIOR0),7	; 1 cycle ; Status update?
	rjmp	trigcheck1d	; 2 cycles
						; 6 cycles
	rjmp	canceltrigd

trigcheck2d:
	in		r26,0x06	; 1 cycle ; data2= Read portB input
/*    inc     r26         ; 1
    inc     r26         ; 1
    inc     r26         ; 1
    lpm*/
	cp		r26,r24		; 1 cycle ; compare triggervalue with data2
	brlo    triggeredd	; 1 cycle  ; data2 < triggervalue ?
	sbis	_SFR_IO_ADDR(GPIOR0),7	; 1 cycle ; Status update?
	rjmp	trigcheck2d	; 2 cycles
						; 6 cycles	

	rjmp	canceltrigd

triggeredd:
	sbi		_SFR_IO_ADDR(GPIOR0), 2  ; Set trigger

canceltrigd:
	pop		r26
    ret
.endfunc


; rising edge trigger detect
;----------------------------------------------------------------------------;
;
;                              .___ data2 
;                              /
;  triggervalue - - - - - - - /
;                  data1 ___./   
;
.global trigup
.func trigup
trigup:     ; input r24: triggervalue
	push	r26			; data
trigcheck1u:
	in		r26,0x06	; 1 cycle ; data1= Read portB input
/*    dec     r26
    dec     r26
    dec     r26
    lpm*/
	cp		r26,r24		; 1 cycle ; compare triggervalue with data1
	brlo    trigcheck2u	; 1 cycle  ; triggervalue < data1 ?
	sbis	_SFR_IO_ADDR(GPIOR0),7	; 1 cycle ; Status update?
	rjmp	trigcheck1u	; 2 cycles
						;; 6 cycles
	rjmp	canceltrigu

trigcheck2u:
	in		r26,0x06	; 1 cycle ; data2= Read portB input
/*    inc     r26
    inc     r26
    inc     r26
    lpm */
	cp		r24,r26		; 1 cycle ; compare triggervalue with data2
	brlo    triggeredu	; 1 cycle  ; data2 < triggervalue ?
	sbis	_SFR_IO_ADDR(GPIOR0),7	; 1 cycle ; Status update?
	rjmp	trigcheck2u	; 2 cycles
						;; 6 cycles	

	rjmp	canceltrigu

triggeredu:
	sbi		_SFR_IO_ADDR(GPIOR0), 2  ; Set trigger

canceltrigu:
	pop		r26
    ret
.endfunc


;----------------------------------------------------------------------------;
; 5uS   / div  ->  312.5nS / pixel ->  3.2 MS / s
.global acquire
.func acquire   ; input: r22:r23, r24:r25
acquire:
    push r28    ; Save YL
    push r29    ; Save YH
    push r24    ; Analog storage
    push r26    ; Digital storage
    push r30    ; Save ZL
    push r31    ; Save ZH
    movw YL, r24
    movw ZL, r22

    ; ADC is delayed 5 samples, so read 5 digital inputs first to be in sync
    in   R26,0x03
    ST   Z+,R26
    lpm				; 3 cycle delay
    in   R26,0x03
    ST   Z+,R26
    lpm
    in   R26,0x03
    ST   Z+,R26
    lpm
    in   R26,0x03
    ST   Z+,R26
    lpm
    in   R26,0x03
    ST   Z+,R26
	; cycles
    in   R24,0x06   ; 1 cycle
    ST   Y+,R24		; 2 cycles
    in   R26,0x03	; 1 cycle
    ST   Z+,R26		; 2 cycles
					; 6 cycles total for one set of samples (analog+digital)
					; 19.2Mhz / 6 cycles = 3.2MHz
    in   R24,0x06   ; 2 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8 sample
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 4
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 5
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 6
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 7
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 8
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 1
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 2
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
    in   R24,0x06   ; 3
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26

    ; Last 5 analog inputs
    in   R24,0x06   ; 4
    ST   Y+,R24
    lpm
    in   R24,0x06   ; 5
    ST   Y+,R24
    lpm
    in   R24,0x06   ; 6
    ST   Y+,R24
    lpm
    in   R24,0x06   ; 7
    ST   Y+,R24
    lpm
    in   R24,0x06   ; 8
    ST   Y+,R24

    pop r31
    pop r30
    pop r26    
    pop r24
    pop r29
    pop r28
	ret
.endfunc

;----------------------------------------------------------------------------;
; 10uS  / div  ->  625nS   / pixel ->  1.6 MS / s
.global acquire2
.func acquire2
acquire2:
    push r28    ; Save YL
    push r29    ; Save YH
    push r24    ; Analog storage
    push r26    ; Digital storage
    push r30    ; Save ZL
    push r31    ; Save ZH
    push r25    ; counter
    movw YL, r24
    movw ZL, r22
    ldi r25, 128

    ; ADC is delayed 5 samples, so read 5 digital inputs first to be in sync
    in   R26,0x03
    ST   Z+,R26
    lpm				; 3 cycle delay
    lpm				; 3 cycle delay
    lpm				; 3 cycle delay
    in   R26,0x03
    ST   Z+,R26
    lpm
    lpm
    lpm
    in   R26,0x03
    ST   Z+,R26
    lpm
    lpm
    lpm
    in   R26,0x03
    ST   Z+,R26
    lpm
    lpm
    lpm
    in   R26,0x03
    ST   Z+,R26
; remaining samples:
more:                   ; cycles:
    in   R24,0x06       ; 1
    ST   Y+,R24         ; 2
    in   R26,0x03       ; 1
    ST   Z+,R26         ; 2
    lpm                 ; 3
    dec  r25            ; 1
    brne more           ; 2
                        ; = 12 cycles total for one set of samples (analog+digital)
						; 19.2Mhz / 12 cycles = 1.6MHz
    pop r25
    pop r31
    pop r30
    pop r26    
    pop r24
    pop r29
    pop r28
	ret
.endfunc

;----------------------------------------------------------------------------;
.global acquire3
.func acquire3
acquire3:
    push r28    ; Save YL
    push r29    ; Save YH
    push r24    ; Analog storage
    push r26    ; Digital storage
    push r30    ; Save ZL
    push r31    ; Save ZH
    push r25    ; counter
    movw YL, r24
    movw ZL, r22
    ldi r25, 136    ; Acquire 135 samples (first 8 are out of sync)

more3:
    in   R24,0x06
    ST   Y+,R24
    in   R26,0x03
    ST   Z+,R26
wait:
    SBIS    0x15,2      ;wait for timer
    RJMP    wait
    SBI     0x15,2
    dec  r25
    brne more3

    pop r25
    pop r31
    pop r30
    pop r26    
    pop r24
    pop r29
    pop r28
	ret
.endfunc
