/*****************************************************************************

MultiKitB: AVR Oscilloscope and Development Kit

Gabotronics C.A.
February 2009

Copyright 2009 Gabriel Anzziani

This program is distributed under the terms of the GNU General Public License 

www.gabotronics.com
email me at: gabriel@gabotronics.com

The Sierpinski Triangle and Hexagon Fractals
The random function is cyclical, if you watch the fractal long enough,
it will clear all the dots for a moment!

The Mandelbrot Fractal uses Fractional numbers to reduce code size and speed
up execution, however, it will limit the zoom in.

*****************************************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay_basic.h>
#include <avr/sleep.h>
#include "mygccdef.h"
#include "multikit.h"
#include "sed1335.h"
#include "asmutil.h"

void mandelbrot(fixed xx0, fixed yy0, fixed w);

void FRACTAL(void) {
    byte fractal=0;
    fixed mx=float2fix(-1.5),my=float2fix(-1),mw=float2fix(0.015625);
    lcd_init(TEXT);
    lcd_write_command(CSR_DOWN); lcd_puts(PSTR("Fractal"));
    CLKPR = 0x80;       // Change clock prescaler
    CLKPR = 0x04;       // Clock prescaler = 16
    signed int x=0, y=-64;
    byte next;
    for(;;) {
        next = random();
        switch(fractal) {
            case 0: // Triangle
                lcd_pixel(x+64,y+64,2);   // Toggle pixel
                if(next<85)         { x=x/2;      y=(y-64)/2; }   // (0,-64)
                else if(next<170)   { x=(x-64)/2; y=(y+63)/2; }  // (-64,63)
                else                { x=(x+63)/2; y=(y+63)/2; }  // (63,63)
            break;
            case 1: // Hexagon
                lcd_pixel(x+64,y+64,2);   // Toggle pixel
                if(next<51)         { x=(-x)/3;      y=(-128-y)/3; }     // (0,-64)
                else if(next<102)   { x=(-128-x)/3;  y=(-32-y)/3; }  // (-64,-16)
                else if(next<153)   { x=(127-x)/3;  y=(-32-y)/3; }  // (63,-16)
                else if(next<204)   { x=(-80-x)/3;   y=(127-y)/3; }  // (-32,63)
                else                { x=(80-x)/3;   y=(127-y)/3; }  // (32,63)
            break;
        }
        // Check key input
        if(testbit(Status, update) && key) {
            clrbit(Status, update);
            lcd_clear_graphics();
            switch(key) {
                case K1: fractal=0; break;
                case K2: fractal=1; break;
                case K3: // Full Speed to draw this fractal!
                        CLKPR = 0x80;   // Change clock prescaler
                        CLKPR = 0x00;   // Clock prescaler = 1
                        fractal=2; break;
                case K4: mw++; mx-=16*mw; my-=16*mw; break;
                case K5: if(mw>1) { mx+=16*mw; my+=16*mw; mw--; } break;
                case K7: return;
            }
            while(key) delay_ms(50);
            if(fractal==2) mandelbrot(mx,my,mw);
        }
    }
}

void mandelbrot(fixed xx0, fixed yy0, fixed w) {
    fixed x2,y2,x0,y0,d,x,y;
    unsigned char i, j, k;
    d = w; // calculate increments
    // repeat on each screen pixel
    y0 = yy0;
    for (i=0; i<128; i++) {
        x0 = xx0;
        for (j=0; j<128; j++) {
            // initialization
            x = x0;
            y = y0;
            k = 0;
            do { // core iteration
                x2 = fixmult(x,x);
                y2 = fixmult(y,y);
                y = fixmult(x,y);
                y = fixmult(float2fix(2),y);

//                x2 = multfix(x,x);
//                y2 = multfix(y,y);
//                y = multfix(x,y);
//                y = multfix(float2fix(2),y);

                y = y + y0;
                x = x2 - y2 + x0;
                k++;
            } while ( (x2 + y2 < float2fix(4))  && ( k < 64));
            // check if the point belongs to the Mandelbrot set
//          if ( k & 2) lcd_pixel(j,i,1);   // draw bands
            if ( k == 64) lcd_pixel(j,i,1);
            x0 += d;    // compute next point x0
        }
        y0 += d;       // compute next y0
    }
}
